from abc import ABC

from saltedge.api.connections import ConnectionAPI
from saltedge.api.countries import CountriesAPI
from saltedge.api.lead.leads import LeadsAPI
from saltedge.api.accounts import AccountAPI
from saltedge.api.lead.sessions import LeadSessionAPI
from saltedge.api.providers import ProviderAPI
from saltedge.api.transactions import TransactionAPI
from saltedge.http import SaltedgeHttpClient


class BaseSaltedgeService(ABC):
    client: SaltedgeHttpClient
    base_url: str = "https://www.saltedge.com/api"

    def __init__(self, client: SaltedgeHttpClient):
        self.client = client
        self.countries = CountriesAPI(client, self.base_url)
        self.providers = ProviderAPI(client, self.base_url)
        self.leads = LeadsAPI(client, self.base_url)
        self.lead_session = LeadSessionAPI(client, self.base_url)
        self.connections = ConnectionAPI(client, self.base_url)
        self.accounts = AccountAPI(client, self.base_url)
        self.transactions = TransactionAPI(client, self.base_url)


class BasePartnerService(BaseSaltedgeService):
    base_url = BaseSaltedgeService.base_url + "/partners/v1"


class PartnersAccountService(BasePartnerService):
    pass


class PartnersPaymentService(BasePartnerService):
    pass



import base64
import json
import time
from enum import Enum
from typing import Any, Iterator

import requests
from cryptography.exceptions import InvalidSignature
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.hazmat.primitives.asymmetric import padding

from saltedge.logging import log


class SaltedgeHttpClient:
    def __init__(
        self, app_id, secret, private_key: bytes, password: str = None
    ):
        self.app_id = app_id
        self.secret = secret

        # with open(private_path, "rb") as private_key:
        self._private_key = serialization.load_pem_private_key(
            private_key, password=password, backend=default_backend()
        )

    class HttpMethod(Enum):
        GET = "GET"
        POST = "POST"
        DELETE = "DELETE"

    @classmethod
    def verify(
        cls, public_key_data: bytes, message: bytes, signature: str
    ) -> bool:
        """
        Verifies the signature on a message.
        :param public_key_data: Bytes, The public key to use to verify the signature.
        :param message: Bytes, The message to verify.
        :param signature: String, The signature on the message.
        :return:
        """
        public_key = serialization.load_pem_public_key(
            public_key_data, backend=default_backend()
        )

        try:
            public_key.verify(
                base64.b64decode(signature),
                message,
                padding.PKCS1v15(),
                hashes.SHA256(),
            )
            return True
        except InvalidSignature as e:
            log.error(
                "Invalid signature: %s",
                e,
                exc_info=True,
                extra={"signature": signature, "_message": message},
            )
            return False

    def get(self, url, query_params: dict = None) -> Iterator[dict]:
        return self._request(
            self.HttpMethod.GET, url, query_params=query_params
        )

    def post(
        self, url, query_params: dict = None, payload: dict = None
    ) -> Iterator[dict]:
        return self._request(
            self.HttpMethod.GET.POST,
            url,
            query_params=query_params,
            payload=payload,
        )

    def delete(self, url, query_params: dict = None) -> dict:
        return next(
            self._request(
                self.HttpMethod.DELETE, url, query_params=query_params
            )
        )

    def _sign(self, message):
        """
        Signs a message.
        :param message: string, Message to be signed.
        :return: string, The signature of the message for the given key.
        """
        return base64.b64encode(
            self._private_key.sign(
                bytes(message.encode("utf8")),
                padding.PKCS1v15(),
                hashes.SHA256(),
            )
        )

    def _generate_signature(
        self, method: str, expire: str, url: str, payload: str | None = None
    ) -> str:
        """
        Generates base64 encoded SHA256 signature of the string given params, signed with the client's private key.
        :param method: Uppercase method of the HTTP request. Example: GET, POST, PATCH, PUT, DELETE, etc.;
        :param expire: request expiration time as a UNIX timestamp in UTC timezone. The Recommended value is 1 minute from now. The maximum value is 1 hour from now.
        :param url: The full requested URL, with all its complementary parameters;
        :param payload: the request post body. Should be left empty if it is a GET request, or the body is empty;
        :return: base64 encoded SHA1 signature
        """
        message = "{expire}|{method}|{url}|".format(
            expire=expire, method=method, url=url
        )

        if payload:
            message += "{payload}".format(payload=payload)

        signature = self._sign(message)
        return signature.decode("utf8")

    def _generate_headers(
        self, method: HttpMethod, url: str, payload: str | None
    ) -> dict:
        expire = self._expires_at(minutes=1)

        headers = {
            "Accept": "application/json",
            "Content-type": "application/json",
            "Expires-at": expire,
            "App-id": self.app_id,
            "Secret": self.secret,
            "Signature": self._generate_signature(
                method.value.upper(), expire, url, payload
            ),
        }
        return headers

    def _expires_at(self, hours=0, minutes=0, seconds=0):
        return str(time.time() + hours * 3600 + minutes * 60 + seconds)

    def _make_single_http_request(
        self,
        method: HttpMethod,
        url: str,
        params: dict[str, Any],
        payload: dict = None,
    ) -> dict:
        log.debug("Fetching %s. Params: %s", url, params)
        serialized_payload = json.dumps(payload) if payload else None
        query_params = (
            "?" + "&".join([f"{k}={v}" for k, v in params.items()])
            if params
            else ""
        )
        headers: dict = self._generate_headers(
            method, url + query_params, serialized_payload
        )
        response = requests.request(
            method=method.value,
            params=params,
            url=url,
            headers=headers,
            data=serialized_payload,
        )
        if response.status_code >= 400:
            msg = f"Request to {url} exited with code {response.status_code}"
            log.error(msg)
            try:
                payload = response.json()
                msg = payload["error"]
            except Exception as e:
                log.warning("No error class defined in payload: %s", str(e))
            raise Exception(msg)
        return response.json()

    def _request(
        self,
        method: HttpMethod,
        url: str,
        payload: dict = None,
        query_params: dict = None,
    ) -> Iterator[dict]:

        resp: dict = self._make_single_http_request(
            method, url, query_params, payload
        )
        yield resp

        while "meta" in resp and resp["meta"]["next_id"]:
            query_params |= {"from_id": resp["meta"]["next_id"]}
            resp = self._make_single_http_request(
                method, url, query_params, payload
            )
            yield resp



import logging

log = logging.getLogger("saltedge")

if not log.handlers:
    handler = logging.StreamHandler()
    formatter = logging.Formatter(
        # name
        "[%(asctime)s][%(levelname)8s][%(module)30.30s]@[%(lineno)5s]$ %(message)s"
    )
    handler.setFormatter(formatter)
    log.addHandler(handler)

    log.setLevel(logging.DEBUG)



from typing import TypedDict, Literal, Iterator

from saltedge.api import BaseAPI, ListAPI


class CountryDTO(TypedDict):
    name: str
    code: str
    refresh_start_time: Literal[
        0,
        1,
        2,
        3,
        4,
        5,
        6,
        7,
        8,
        9,
        10,
        11,
        12,
        13,
        14,
        15,
        16,
        17,
        18,
        19,
        20,
        21,
        22,
        23,
    ]


class CountriesAPI(ListAPI[CountryDTO]):
    _path = "countries"
    _response_dto_class = CountryDTO

    def list(self, country_code: str = None, **kwargs) -> Iterator[CountryDTO]:
        if country_code:
            kwargs |= {"country_code": country_code}
        return super().list(**kwargs)



from typing import TypedDict, Literal, Iterator

from saltedge.api import ListAPI, RetrieveAPI, BaseAPI, RP
from saltedge.api.attempts import AttemptDTO


class ConnectionDTO(TypedDict):
    id: str
    provider_id: str
    provider_code: str
    provider_name: str
    daily_refresh: bool
    customer_id: str
    created_at: str
    updated_at: str
    last_success_at: str
    status: Literal["active", "inactive", "disabled"]
    country_code: str
    next_refresh_possible_at: str
    last_attempt: AttemptDTO
    show_consent_confirmation: bool
    last_consent_id: str


class ConnectionAPI(
    ListAPI[ConnectionDTO], RetrieveAPI[ConnectionDTO], BaseAPI
):
    _path = "connections"
    _response_dto_class = ConnectionDTO

    def list(self, customer_id: str, **kwargs) -> Iterator[ConnectionDTO]:
        return super().list(customer_id=customer_id, **kwargs)
    
    def show(self, connection_id: str, *args, **kwargs) -> ConnectionDTO:
        return super().show(connection_id)



from typing import TypedDict, Literal, NotRequired


class StageDTO(TypedDict):
    created_at: str
    id: str
    name: Literal[
        "start",
        "connect",
        "interactive",
        "fetch_holder_info",
        "fetch_accounts",
        "fetch_recent",
        "fetch_full",
        "disconnect",
        "finish",
    ]
    updated_at: str


class AttemptDTO(TypedDict):
    api_mode: Literal["app", "service"]
    api_version: str
    automatic_fetch: bool
    daily_refresh: bool
    categorization: NotRequired[Literal["none", "personal", "business"]]
    created_at: str
    custom_fields: dict
    device_type: Literal["desktop", "tablet", "mobile"]
    remote_ip: str
    customer_last_logged_at: str
    fail_at: str
    fail_error_class: str
    fail_message: str
    fetch_scopes: list[
        Literal[
            "accounts",
            "holder_info",
            "transactions",
            "accounts_without_balance",
        ]
    ]
    finished: bool
    finished_recent: bool
    from_date: str
    id: str
    interactive: bool
    locale: Literal[
        "bg",
        "cz",
        "de",
        "en",
        "es-MX",
        "es",
        "fi",
        "fr",
        "he",
        "hr",
        "hu",
        "it",
        "nl",
        "pl",
        "pt-BR",
        "pt",
        "ro",
        "ru",
        "si",
        "sk",
        "sv",
        "tr",
        "uk",
        "zh-HK",
        "zh",
    ]
    partial: bool
    success_at: str
    to_date: str
    updated_at: str
    show_consent_confirmation: bool
    include_natures: list[str]
    stages: list[StageDTO]
    unduplication_strategy: Literal[
        "mark_as_pending", "mark_as_duplicate", "delete_transactions"
    ]



import datetime as dt
from typing import TypedDict, Literal, Any

from saltedge.api import ListAPI, BaseAPI, RetrieveAPI, CreateAPI, BaseAPI


class ProviderDTO(TypedDict):
    id: str
    code: str
    name: str
    mode: Literal["oauth", "web", "api", "file"]
    status: Literal["active", "inactive", "disabled"]
    automatic_fetch: bool
    dynamic_registration_code: str
    group_code: str
    group_name: str
    hub: str
    customer_notified_on_sign_in: bool
    interactive: bool
    instruction: str
    home_url: str
    login_url: str
    logo_url: str
    country_code: str
    refresh_timeout: int
    holder_info: list
    max_consent_days: int
    created_at: dt.datetime
    updated_at: dt.datetime
    timezone: str
    max_interactive_delay: int
    optional_interactivity: bool
    regulated: bool
    max_fetch_interval: int
    custom_pendings: bool
    custom_pendings_period: int
    supported_fetch_scopes: list[str]
    supported_account_extra_fields: list[str]
    supported_transaction_extra_fields: list[str]
    supported_account_natures: list[str]
    supported_account_types: Literal["personal", "business"]
    identification_codes: list[str]
    bic_codes: list[str]
    supported_iframe_embedding: Literal["true", "false"]
    payment_templates: list[str]
    supported_payment_fields: dict[str, Any]
    required_payment_fields: dict[str, Any]
    no_funds_rejection_supported: bool


class ProviderAPI(ListAPI[ProviderDTO], RetrieveAPI[ProviderDTO], BaseAPI):
    _path = "providers"
    _response_dto_class = ProviderDTO

    def show(self, code: str = None, *args, **kwargs) -> ProviderDTO:
        return super().show(code)




from abc import ABC
from typing import (
    Iterator,
    TypeVar,
    Generic,
    TypedDict,
    Any,
    Union,
    Iterable,
    Type,
)

from saltedge.http import SaltedgeHttpClient

# RQ = TypeVar("RQ")  # Request Payload
RP = TypeVar("RP", bound=TypedDict)  # Response payload


class BaseAPI(ABC, Generic[RP]):
    _path: str
    _client: SaltedgeHttpClient
    _base_url: str
    _response_dto_class: Type[RP]

    def __init__(
        self,
        client: SaltedgeHttpClient,
        base_url: str,
    ):
        self._client = client
        self._base_url = base_url


# class BaseRequestResponseAPI(BaseAPI, Generic[RQ]):
#     _request_dto_class: Type[RQ]


class ListAPI(BaseAPI, Generic[RP]):
    # _list_dto_class: Type[RP]

    def list(self, **kwargs) -> Iterator[RP]:
        # dto_class = (
        #     self._list_dto_class
        #     if hasattr(self, "_list_dto_class")
        #     else self._response_dto_class
        # )
        dto_class = self._response_dto_class
        query_params = kwargs or dict()
        for payload in self._client.get(
            f"{self._base_url}/{self._path}", query_params=query_params
        ):
            for data in payload["data"]:
                yield dto_class(**data)


class RetrieveAPI(BaseAPI, Generic[RP]):
    # _retrieve_dto_class: Type[RP]

    def show(self, *args, **kwargs) -> RP:
        # dto_class = (
        #     self._retrieve_dto_class
        #     if hasattr(self, "_retrieve_dto_class")
        #     else self._response_dto_class
        # )
        dto_class = self._response_dto_class
        path_params = [arg for arg in args if arg is not None]
        path_params_str = ""
        if path_params:
            path_params_str = "/" + ", ".join(path_params)
        query_params = kwargs or dict()
        resp = list(
            self._client.get(
                f"{self._base_url}/{self._path}{path_params_str}",
                query_params=query_params,
            )
        )[0]
        return dto_class(**resp["data"])


class CreateAPI(BaseAPI, Generic[RP]):
    # _create_dto_class: Type[RP]

    def create(self, payload: dict, *args, **kwargs) -> RP:
        # dto_class = (
        #     self._create_dto_class
        #     if hasattr(self, "_create_dto_class")
        #     else self._request_dto_class
        # )
        dto_class = self._response_dto_class
        payload = list(
            self._client.post(
                f"{self._base_url}/{self._path}/", payload={"data": payload}
            )
        )[0]
        return dto_class(**payload["data"])
        # for data in payload["data"]:
        #     yield dto_class(**data)


class DeleteAPI(BaseAPI, Generic[RP]):
    def delete(self, *args, **kwargs) -> RP:
        dto_class = self._response_dto_class
        query_params = kwargs or dict()
        payload = self._client.delete(
            f"{self._base_url}/{self._path}", query_params=query_params
        )
        return dto_class(**payload["data"])



from typing import TypedDict, Iterator

from saltedge.api import ListAPI, RetrieveAPI, BaseAPI, RP


class CustomerDTO(TypedDict):
    id: str
    identifier: str
    created_at: str
    updated_at: str


class CustomerAPI(ListAPI[CustomerDTO], RetrieveAPI[CustomerDTO], BaseAPI):
    _path = "customers"
    _request_dto_class = CustomerDTO

    def show(self, customer_id: int) -> CustomerDTO:
        return super().show(customer_id)



import datetime as dt
from typing import TypedDict, NotRequired, Literal

from saltedge.api import CreateAPI, DeleteAPI, BaseAPI


class KycDTO(TypedDict):
    full_name: NotRequired[str]
    type_of_account: NotRequired[Literal["own", "legal", "shared"]]
    citizenship_code: NotRequired[str]
    residence_address: NotRequired[str]
    date_of_birth: NotRequired[dt.date]
    gender: NotRequired[Literal["male", "female", "other"]]
    legal_name: NotRequired[str]
    registered_office_code: NotRequired[str]
    registered_office_address: NotRequired[str]
    registration_number: NotRequired[str]


class CreateLeadDTO(TypedDict):
    email: str
    identifier: NotRequired[str]
    kyc: NotRequired[KycDTO]


class CreatedLeadResponseDTO(TypedDict):
    email: str
    customer_id: str
    identifier: str


class DeletedLeadResponseDTO(TypedDict):
    deleted: bool
    lead_email: str


class LeadsAPI:
    class CreateLeadsAPI(
        CreateAPI[CreatedLeadResponseDTO], BaseAPI
    ):
        _path = "leads"
        _response_dto_class = CreatedLeadResponseDTO

        # def create(self, payload: CreateLeadDTO, *args, **kwargs) -> CreatedLeadResponseDTO:
        #     return super().create(payload)

    class DeleteLeadsAPI(DeleteAPI[DeletedLeadResponseDTO], BaseAPI):
        _path = "leads"
        _response_dto_class = DeletedLeadResponseDTO

    def __init__(self, client, base_url):
        super().__init__()
        self._create_api = self.CreateLeadsAPI(client, base_url=base_url)
        self._delete_api = self.DeleteLeadsAPI(client, base_url=base_url)

    def create(self, payload: CreateLeadDTO) -> CreatedLeadResponseDTO:
        return self._create_api.create(payload=payload)

    def delete(self, customer_id: str) -> DeletedLeadResponseDTO:
        return self._delete_api.delete(customer_id=customer_id)



from typing import TypedDict, Literal, NotRequired, Iterator

from saltedge.api import ListAPI, BaseAPI, RP
from saltedge.http import SaltedgeHttpClient


class TransactionExtraDTO(TypedDict):
    account_balance_snapshot: NotRequired[float]
    account_number: NotRequired[str]
    additional: NotRequired[str]
    asset_amount: NotRequired[float]
    asset_code: NotRequired[str]
    categorization_confidence: NotRequired[float]
    check_number: NotRequired[str]
    closing_balance: NotRequired[float]
    constant_code: NotRequired[str]
    convert: NotRequired[bool]
    customer_category_code: NotRequired[str]
    customer_category_name: NotRequired[str]
    exchange_rate: NotRequired[float]
    id: NotRequired[str]
    end_to_end_id: NotRequired[str]
    information: NotRequired[str]
    mcc: NotRequired[str]
    merchant_id: NotRequired[str]
    opening_balance: NotRequired[float]
    installment_debt_amount: NotRequired[float]
    original_amount: NotRequired[float]
    original_category: NotRequired[str]
    original_currency_code: NotRequired[str]
    original_subcategory: NotRequired[str]
    payee: NotRequired[str]
    payee_information: NotRequired[str]
    payer: NotRequired[str]
    payer_information: NotRequired[str]
    possible_duplicate: NotRequired[bool]
    posted_by_aspsp: NotRequired[bool]
    posting_date: NotRequired[str]
    posting_time: NotRequired[str]
    record_number: NotRequired[str]
    specific_code: NotRequired[str]
    tags: NotRequired[list[str]]
    time: NotRequired[str]
    transfer_account_name: NotRequired[str]
    type: NotRequired[str]
    unit_price: NotRequired[float]
    units: NotRequired[float]
    variable_code: NotRequired[str]


class TransactionDTO(TypedDict):
    id: str
    mode: Literal["normal", "fee", "transfer"]
    status: Literal["posted", "pending"]
    made_on: str
    amount: float
    currency_code: str
    description: str
    category: str
    duplicated: bool
    extra: NotRequired[TransactionExtraDTO]
    account_id: str
    created_at: str
    updated_at: str


class TransactionAPI:
    class TransactionListAPI(ListAPI[TransactionDTO], BaseAPI):
        _path = "transactions"
        _response_dto_class = TransactionDTO

        def list(
            self, connection_id: str, account_id: str = None, **kwargs
        ) -> Iterator[TransactionDTO]:
            params = {"connection_id": connection_id}
            if account_id:
                params |= {"account_id": account_id}
            return super().list(**params)

    class TransactionPendingAPI(TransactionListAPI):
        _path = "transactions/pending"

    class TransactionDuplicatesAPI(TransactionListAPI):
        _path = "transactions/duplicates"

    def __init__(self, client: SaltedgeHttpClient, base_url: str):
        self._transactions = self.TransactionListAPI(client, base_url)
        self._pending = self.TransactionPendingAPI(client, base_url)
        self._duplicates = self.TransactionDuplicatesAPI(client, base_url)

    def list(self, connection_id: str, account_id: str = None):
        return self._transactions.list(connection_id, account_id=account_id)

    def pending(self, connection_id: str, account_id: str = None):
        return self._pending.list(connection_id, account_id=account_id)

    def duplicates(self, connection_id: str, account_id: str = None):
        return self._duplicates.list(connection_id, account_id=account_id)



from typing import TypedDict, Literal, NotRequired

from saltedge.api import CreateAPI, BaseAPI
from saltedge.logging import log


class KycDTO(TypedDict):
    type_of_account: NotRequired[Literal["own", "legal", "shared"]]


ALL_CONSENTS = [
    "account_details",
    # "holder_information",
    "transactions_details",
]


class ConsentDTO(TypedDict):
    # valid combinations are:
    # ["account_details"],
    # ["holder_information"],
    # ["account_details", "holder_information"],
    # ["account_details", "transactions_details"],
    # ["account_details", "holder_information", "transactions_details"],
    scopes: list[
        Literal[
            "account_details", "holder_information", "transactions_details"
        ]
    ]
    from_date: NotRequired[str]  # 2024-03-01, with 365 days ago
    to_date: NotRequired[str]  # 2024-03-01
    period_days: NotRequired[int]


class AttemptDTO(TypedDict):
    # valid combinations are:
    # ['accounts'],
    # ['holder_info'],
    # ['accounts', 'holder_info'],
    # ['accounts', 'transactions'],
    # ['accounts', 'holder_info', 'transactions'],
    # ['accounts_without_balance'],
    # ['accounts_without_balance', 'holder_info']
    fetch_scopes: NotRequired[
        list[
            Literal[
                "accounts",
                "holder_info",
                "transactions",
                "accounts_without_balance",
            ]
        ]
    ]
    from_date: NotRequired[str]  # 2024-03-01, with 365 days ago
    to_date: NotRequired[str]  # 2024-03-01
    fetched_accounts_notify: NotRequired[bool]
    custom_fields: NotRequired[dict]
    locale: NotRequired[
        Literal[
            "bg",
            "cz",
            "de",
            "en",
            "es-MX",
            "es",
            "fi",
            "fr",
            "he",
            "hr",
            "hu",
            "it",
            "nl",
            "pl",
            "pt-BR",
            "pt",
            "ro",
            "ru",
            "si",
            "sk",
            "sv",
            "tr",
            "uk",
            "zh-HK",
            "zh",
        ]
    ]
    include_natures: NotRequired[
        list[
            Literal[
                "account",
                "bonus",
                "card",
                "checking",
                "credit",
                "credit_card",
                "debit_card",
                "ewallet",
                "insurance",
                "investment",
                "loan",
                "mortgage",
                "savings",
            ]
        ]
    ]
    customer_last_logged_at: NotRequired[str]  # datetime
    unduplication_strategy: NotRequired[
        Literal["mark_as_pending", "mark_as_duplicate", "delete_transactions"]
    ]
    return_to: NotRequired[str]


class LeadSessionDTO(TypedDict):
    customer_id: NotRequired[str]
    consent: ConsentDTO
    attempt: NotRequired[AttemptDTO]
    allowed_countries: NotRequired[
        list[
            Literal[
                "AT",
                "BE",
                "BG",
                "CY",
                "CZ",
                "DE",
                "DK",
                "EE",
                "ES",
                "FI",
                "FR",
                "GB",
                "GR",
                "HR",
                "HU",
                "IE",
                "IS",
                "IT",
                "LI",
                "LT",
                "LU",
                "LV",
                "MT",
                "NL",
                "NO",
                "PL",
                "PT",
                "RO",
                "SE",
                "SI",
                "SK",
                "XF",
            ]
        ]
    ]
    country_code: NotRequired[str]
    provider_code: NotRequired[str]
    skip_provider_select: NotRequired[bool]
    daily_refresh: NotRequired[bool]
    disable_provider_search: NotRequired[bool]
    return_connection_id: NotRequired[bool]
    provider_modes: NotRequired[list[Literal["oauth", "web", "api", "file"]]]
    categorization: NotRequired[list[Literal["none", "personal", "business"]]]
    include_fake_providers: NotRequired[bool]
    return_error_class: NotRequired[bool]
    regulated: NotRequired[bool]
    theme: NotRequired[Literal["default", "dark"]]
    connect_template: NotRequired[str]
    javascript_callback_type: NotRequired[Literal["post_message"]]
    kyc: NotRequired[KycDTO]


class LeadSessionCreatedResponseDTO(TypedDict):
    expires_at: str
    redirect_url: str


class LeadSessionCreateAPI(CreateAPI[LeadSessionCreatedResponseDTO]):
    _path = "lead_sessions/create"
    _response_dto_class = LeadSessionCreatedResponseDTO

    def create(
        self, payload: LeadSessionDTO, *args, **kwargs
    ) -> LeadSessionCreatedResponseDTO:
        return super().create(payload)


class LeadSessionAPI:
    def __init__(self, client, base_url):
        self._create_api = LeadSessionCreateAPI(client, base_url=base_url)

    def create(self, payload: LeadSessionDTO) -> LeadSessionCreatedResponseDTO:
        log.debug("Create a lead")
        return self._create_api.create(payload=payload)

    def reconnect(self):
        raise NotImplementedError()

    def refresh(self):
        raise NotImplementedError()



import datetime as dt
from typing import Literal, TypedDict, Iterator

from saltedge.api import ListAPI, BaseAPI, RP


class FloatingInterestRateDTO(TypedDict):
    min_value: float
    max_value: float


class AccountExtraDTO(TypedDict):
    account_name: str
    account_number: str
    assets: list[dict]
    available_amount: float
    balance_type: str
    blocked_amount: float
    card_type: Literal[
        "american_express",
        "china_unionpay",
        "diners_club",
        "jcb",
        "maestro",
        "master_card",
        "uatp",
        "visa",
        "mir",
    ]
    cards: list[str]
    client_name: str
    closing_balance: float
    credit_limit: float
    current_date: dt.date
    current_time: dt.datetime
    expiry_date: dt.date
    iban: str
    bban: str
    interest_rate: float
    interest_type: str
    floating_interest_rate: FloatingInterestRateDTO
    remaining_payments: float
    penalty_amount: float
    next_payment_amount: float
    next_payment_date: dt.date
    open_date: dt.date
    opening_balance: float  # Assuming opening_balance also uses float
    partial: bool
    provider_account_id: str
    raw_balance: str
    sort_code: str
    statement_cut_date: dt.date
    status: str
    swift: str
    total_payment_amount: dt.date
    transactions_count: dict
    payment_type: str
    cashback_amount: float


class AccountDTO(TypedDict):
    id: str
    name: str
    nature: Literal[
        "account",
        "bonus",
        "card",
        "checking",
        "credit",
        "credit_card",
        "debit_card",
        "ewallet",
        "insurance",
        "investment",
        "loan",
        "mortgage",
        "savings",
    ]
    balance: float
    currency_code: str
    extra: AccountExtraDTO
    connection_id: str
    created_at: str
    updated_at: str


class AccountAPI(ListAPI[AccountDTO], BaseAPI):
    _path = "accounts"
    _response_dto_class = AccountDTO

    def list(
        self, connection_id: str = None, customer_id: str = None
    ) -> Iterator[AccountDTO]:
        params = dict()
        match (connection_id, customer_id):
            case (None, None):
                raise ValueError(
                    "Connection_id and customer_id can't be both None"
                )
            case x, None if x is not None:
                params["connection_id"] = x
            case None, y if y is not None:
                params["customer_id"] = y
            case x, y if x is not None and y is not None:
                params["connection_id"] = x
        return super().list(**params)
